///////////////////////////////////////////////////////////////////////////////////////////////////
// assem.cpp 
// This is the main module for the assembler application.
//
// World III
//
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci 
// Revised : 11 May 2007
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
///////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
// Defines
///////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////////////////////////
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <curses.h>
#include <strings.h>
#include <sys/timeb.h>
#include <pthread.h>

#include "wiii_curses.h"
#include "wiii_util.h"
#include "wiii_structures.h"
#include "cstring.hpp"
#include "cengine.hpp"
#include "worldiii.h"

///////////////////////////////////////////////////////////////////////////////////////////////////
// Globals
///////////////////////////////////////////////////////////////////////////////////////////////////

bool 			debug=false;
int			menucontext;
FILE 			*logfile;
pthread_attr_t		attr;                           // thread attr structure for setting attributes.
pthread_mutexattr_t	m_attr;
int			loglevel;
struct timeb		starttime;
long			logcount;
pthread_mutex_t		screenmutex;
pthread_mutex_t		logmutex;
int			currentlistcount;		// how many lists are there
CEngine			*engine;
CString			inputfilename;
CString			outputfile;
CString			exefile;
CString			lstfile;
bool			cmdline;			// are we running from the command line, or 
							// are we running interactive
bool			lst, exe, assem, curs;

////////////////////////////////////////////////////////////////////////////////////////////////////
// Menu Def
// Menu moves are not actions
////////////////////////////////////////////////////////////////////////////////////////////////////
typedef struct menutype
{
	int	context;	// which sub menu are we in?
	char	option;		// option letter to type in 
	char	*description;	// description of that option
	bool	action;		// Is this option a sub menu or an action, 1=action, 0=submenu ?
	int	next;		// the next menu context
	} menu_t;

#define MAIN_CONTEXT	0   
#define FILE_CONTEXT	1   
#define TEST_CONTEXT	2   
//#define NAV_CONTEXT	3   
#define HELP_CONTEXT	4   
//#define NEW_CONTEXT	5   
//#define DEL_CONTEXT	6   
#define ASM_CONTEXT	7   
#define COUNT   25

//	context		opt	description		action		parent/child context
menu_t	menu[]=
{
	{MAIN_CONTEXT,	'f',	"File               ",	0,		FILE_CONTEXT},
	{MAIN_CONTEXT,	'a',	"Assemble           ",	0,		ASM_CONTEXT},
	{MAIN_CONTEXT,	't',	"Test Script        ",	0,		TEST_CONTEXT},
	{MAIN_CONTEXT,	'h',	"Help               ",	0,		HELP_CONTEXT},
	{MAIN_CONTEXT,	'z',	"Exit Program       ",	1,		MAIN_CONTEXT},

	{FILE_CONTEXT, 	'o',	"Open               ",	1,		MAIN_CONTEXT},
	{FILE_CONTEXT, 	's',	"Save               ",	1,		MAIN_CONTEXT},
	{FILE_CONTEXT, 	'e',	"Exit               ",	1,		MAIN_CONTEXT},
	{FILE_CONTEXT, 	'z',	"Previous Menu      ",	0,		MAIN_CONTEXT},

	{ASM_CONTEXT, 	'o',	"Open for assembly  ",	1,		MAIN_CONTEXT},
	{ASM_CONTEXT, 	'f',	"Do First Pass      ",	1,		MAIN_CONTEXT},
	{ASM_CONTEXT, 	's',	"Do Second Pass     ",	1,		MAIN_CONTEXT},
	{ASM_CONTEXT, 	'l',	"Log Binary to File ",	1,		MAIN_CONTEXT},
	{ASM_CONTEXT, 	'w',	"Write Bin          ",	1,		MAIN_CONTEXT},
	{ASM_CONTEXT, 	'z',	"Previous Menu      ",	0,		MAIN_CONTEXT},

	{HELP_CONTEXT, 	'a',	"About              ",	1,		MAIN_CONTEXT},
	{HELP_CONTEXT, 	'h',	"Help               ",	1,		MAIN_CONTEXT},
	{HELP_CONTEXT, 	'z',	"Previous Menu      ", 	0,		MAIN_CONTEXT},

	{TEST_CONTEXT, 	's',	"Single Step        ",	1,		MAIN_CONTEXT},
	{TEST_CONTEXT, 	'r',	"Run                ",	1,		MAIN_CONTEXT},
	{TEST_CONTEXT, 	'd',	"Display Registers  ",	1,		MAIN_CONTEXT},
	{TEST_CONTEXT, 	'R',	"Reset              ",	1,		MAIN_CONTEXT},
	{TEST_CONTEXT, 	'p',	"Set PC             ",	1,		MAIN_CONTEXT},
	{TEST_CONTEXT, 	'l',	"List Mem           ",	1,		MAIN_CONTEXT},
	{TEST_CONTEXT, 	'z',	"Previous Menu      ", 	0,		MAIN_CONTEXT},

	};


///////////////////////////////////////////////////////////////////////////////////////////////////
// External Functions
///////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
// Forward Decs
///////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
// ParseCommandLine
////////////////////////////////////////////////////////////////////////////////////////////////////
void ParseCommandLine(int argc, char** argv)
{
int     i;

        // set defaults for parms
        loglevel=0;
	cmdline=false;

        //getting parms
        for (i=0;i<argc;i++)
        {

                // --loglevel how much logging to do
                if ((strncmp(argv[i],"--loglevel",10))==0)
                {
                        printf("DEBUGMODE ON\n");
                        loglevel=atoi(argv[i+1]);
                        i++;
                        }

                // -a assemble from the command line
                if ((strncmp(argv[i],"-a",2))==0)
                {
			if (argc>i+1)
			{
				inputfilename.SetString(argv[i+1]);
        	                i++;
				cmdline=true;
				assem=true;
				}
			else 
			{
				printf("Error not enough parameters\n");
				exit (1);
				}
                        }

		// -o outputfilename from the command line
		if ((strncmp(argv[i],"-o",2))==0)
		{
			if (argc>i+1)
			{
				outputfile.SetString(argv[i+1]);
				i++;
				cmdline=true;
				assem=true;
				}
			else 
			{
				printf("Error not enough parameters\n");
				exit (1);
				}
			}

		// -e script.bin from the command line
		if ((strncmp(argv[i],"-e",2))==0)
		{
			if (argc>i+1)
			{
				exefile.SetString(argv[i+1]);
				i++;
				cmdline=true;
				exe=true;
				}
			else 
			{
				printf("Error not enough parameters\n");
				exit (1);
				}
			}

		// -l script.lst from the command line
		if ((strncmp(argv[i],"-l",2))==0)
		{
			if (argc>i+1)
			{
				lstfile.SetString(argv[i+1]);
				i++;
				cmdline=true;
				lst=true;
				}
			else 
			{
				printf("Error not enough parameters\n");
				exit (1);
				}
			}
                } //end for loop

	// check options
	if (exe&&assem) 
	{
		printf("Error: Can't assemble and execute at same time.\n");
		exit (1);
		}
	
	if (exe&&lst) 
	{
		printf("Error: Can't list and execute at same time.\n");
		exit (1);
		}
	
        } //////////////////////////////////////////////////////////////////////end ParseCommandLine

///////////////////////////////////////////////////////////////////////////////////////////////////
// Init
///////////////////////////////////////////////////////////////////////////////////////////////////
void Init(int argc, char** argv)
{
//int i;

	// Parse input options
	ParseCommandLine(argc,argv);

        // print out parms if requested
        if (loglevel>0)
        {
        int i;

                for (i=0;i<argc;i++) printf("Argument %d, s='%s' \n",i,argv[i]);
                sleep (2);
                }

        pthread_attr_init(&attr);    // initialize attr with default attributes
        pthread_attr_setscope(&attr, PTHREAD_SCOPE_SYSTEM);  // system-wide contention

        InitMutex(&screenmutex);
        InitMutex(&logmutex);

	} //////////////////////////////////////////////////////////////////////////////////end Init

////////////////////////////////////////////////////////////////////////////////////////////////////
// InitSock
////////////////////////////////////////////////////////////////////////////////////////////////////
void InitSock()
{

     } /////////////////////////////////////////////////////////////////////////////////end InitSock

////////////////////////////////////////////////////////////////////////////////////////////////////
// DeInit
////////////////////////////////////////////////////////////////////////////////////////////////////
void DeInit()
{
     // 

     } ///////////////////////////////////////////////////////////////////////////////////end DeInit

////////////////////////////////////////////////////////////////////////////////////////////////////
// InitScreen
////////////////////////////////////////////////////////////////////////////////////////////////////
void InitScreen(curswin **p)
{
	// init the ncurses library
	initscr();
	cbreak();
	noecho();
	nonl();
	intrflush(stdscr, false);
	keypad(stdscr, true);
	
	// get mem for curswin
	(*p) = new curswin;

	// create the windows
	(*p)->menuwin =newwin(int(LINES/2),int(COLS/2), 0,0);
	(*p)->debugwin=newwin(int(LINES/2),int(COLS/2),0,int(COLS/2));
	(*p)->mainwin =newwin(int(LINES/2),COLS,int(LINES/2),0);
	scrollok((*p)->menuwin, true);
	scrollok((*p)->debugwin,true);
	scrollok((*p)->mainwin, true);
	menucontext=MAIN_CONTEXT;
	} ////////////////////////////////////////////////////////////////////////////end InitScreen

////////////////////////////////////////////////////////////////////////////////////////////////////
// DeInitScreen
////////////////////////////////////////////////////////////////////////////////////////////////////
void DeInitScreen(curswin *p)
{
	// DeInit the ncurses library
	wrefresh(p->menuwin);
	wrefresh(p->debugwin);
	wrefresh(p->mainwin);
	endwin();
	} //////////////////////////////////////////////////////////////////////////end DeInitScreen

////////////////////////////////////////////////////////////////////////////////////////////////////
// PrintMenu
////////////////////////////////////////////////////////////////////////////////////////////////////
char PrintMenu(int *context, curswin *cw)
{
int	i,p;
char	buf[80];
char	c;


PrintMenu0:
	// print menu
	werase(cw->menuwin);
	p=0;
	for (i=0;i<COUNT;i++)
	{
		if (menu[i].context==*context) 
		{
			snprintf(buf,80,"'%c' = %s",menu[i].option,menu[i].description);
			MyLocPrint(cw->menuwin,p,0,buf);
			p++;
			}
		}

	// get choice
	c=(char)getch();

	// check for exit condition
	if ((*context==MAIN_CONTEXT)&&(c=='z')) return ('z');

	// see if we need to change menu context
	for (i=0;i<COUNT;i++)	
	if ((menu[i].context==*context)&&(c==menu[i].option)&&!menu[i].action)
	{
		*context=menu[i].next;
		goto PrintMenu0;
		}

	// return action choice
	return (c);
	
	} /////////////////////////////////////////////////////////////////////////////end Printmenu

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetAnInt
////////////////////////////////////////////////////////////////////////////////////////////////////
int GetAnInt(curswin *p)
{
CString		myinput;
char		c1[2];
char		buf[80];

	c1[0]='\0';
	c1[1]='\0';
	while (c1[0]!=13)
	{
		c1[0]=(char)getch();
		c1[1]='\0';
		MyPrint(p->mainwin,c1,true);
		if (c1[0]!=13) myinput.Append(c1);
		if (debug) snprintf(buf,80,"String is '%s' \n",myinput.GetStringPtr());
		if (debug) MyPrint(p->debugwin,buf,true);
		}
	return (atoi(myinput.GetStringPtr()));
	} //////////////////////////////////////////////////////////////////////////////end GetAnInt

////////////////////////////////////////////////////////////////////////////////////////////////////
// PrintCurr
////////////////////////////////////////////////////////////////////////////////////////////////////
void PrintCurr(curswin *p)
{
char	buf[80];

	MyPrint(p->debugwin,"Print Object \n", true);
//	snprintf(buf,80,"value is %d \n",(list->GetCurrent())->GetVal());
	MyPrint(p->mainwin,buf,true);
	} /////////////////////////////////////////////////////////////////////////////end PrintCurr

////////////////////////////////////////////////////////////////////////////////////////////////////
// CEPrintInt
// CEngine external function
// Takes single int
// No returns
// Prints integer
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEPrintInt(parmstruct *p)
{
char	buf[80];
curswin	*cw;

	// These are simple external routines for the scripting engine
	cw=(curswin*)(p->extra);
	if (cw==0) return;
	snprintf(buf,80,"Print from script = %li\n",p->i[0]);
	MyPrint(cw->mainwin,buf,true);
	} ////////////////////////////////////////////////////////////////////////////end CEPrintInt

////////////////////////////////////////////////////////////////////////////////////////////////////
// CEPrintCString
// CEngine external function
// Takes single CString
// No returns
// Prints CString
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEPrintCString(parmstruct *p)
{
char	buf[80];
curswin	*cw;

	// These are simple external routines for the scripting engine
	cw=(curswin*)(p->extra);
	if (cw==0) return;
	p->s[0].ConvertBinary();
	snprintf(buf,80,"Print from script = '%s'\n",p->s[0].GetStringPtr());
	MyPrint(cw->mainwin,buf,true);
	} ////////////////////////////////////////////////////////////////////////end CEPrintCString

////////////////////////////////////////////////////////////////////////////////////////////////////
// CEPrintf
// CEngine external function
// Takes single CString
// No returns
// Prints CString
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEPrintf(parmstruct *p)
{
char	buf[80];

	// These are simple external routines for the scripting engine
	snprintf(buf,80,"%s",p->s[0].GetStringPtr());
	printf(buf);
	} //////////////////////////////////////////////////////////////////////////////end CEPrintf

////////////////////////////////////////////////////////////////////////////////////////////////////
// CEPrintfU
// CEngine external function
// Takes single uint
// No returns
// Prints CString
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEPrintfU(parmstruct *p)
{
char	buf[80];

	// These are simple external routines for the scripting engine
	snprintf(buf,80,"%lu\n",p->u[0]);
	printf(buf);
	} /////////////////////////////////////////////////////////////////////////////end CEPrintfU

////////////////////////////////////////////////////////////////////////////////////////////////////
// CEPrintfI
// CEngine external function
// Takes single int
// No returns
// Prints CString
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEPrintfI(parmstruct *p)
{
char	buf[80];

	// These are simple external routines for the scripting engine
	snprintf(buf,80,"%li\n",p->i[0]);
	printf(buf);
	} /////////////////////////////////////////////////////////////////////////////end CEPrintfI

////////////////////////////////////////////////////////////////////////////////////////////////////
// TestFile
////////////////////////////////////////////////////////////////////////////////////////////////////
void TestFile(int menucontext, char c)
{
	// ok read the file until no more input and execute the test.
	
	} //////////////////////////////////////////////////////////////////////////////end TestFile

////////////////////////////////////////////////////////////////////////////////////////////////////
// ListScript
////////////////////////////////////////////////////////////////////////////////////////////////////
void ListScript(unsigned long int start, curswin *p)
{
unsigned long int       li;
char                    buf[BUFFSIZE];

	MyPrint(p->mainwin,"Listing script  \n", TRUE);
	for (li=start;li<start+10;li++)
	{
		snprintf(buf,BUFFSIZE,"%lu     %lu\n",li,engine->GetValue(li));
		MyPrint(p->mainwin,buf, TRUE);
		}
	} ////////////////////////////////////////////////////////////////////////////end ListScript

////////////////////////////////////////////////////////////////////////////////////////////////////
// Display
////////////////////////////////////////////////////////////////////////////////////////////////////
void Display(curswin *p)
{
int i;
bool b[10],eq,gt,lt,cy;
unsigned long int pc,sp;
long int ints[10];
unsigned long int uints[10];
double dbls[10];
CString s[10];
char buf[BUFFSIZE];


	MyPrint(p->debugwin,"Display registers ", TRUE);
	engine->GetState(&pc,&sp,&eq,&gt,&lt,&cy);
	snprintf(buf,BUFFSIZE,"pc=%lu sp=%lu eq=%d gt=%d lt=%d  cy=%d\n", pc,sp,eq,gt,lt,cy);
	MyPrint(p->debugwin,buf, TRUE);

	MyPrint(p->debugwin,"Booleans ", TRUE);
	engine->GetBools(b);
	snprintf(buf,BUFFSIZE,"%d %d %d %d %d %d %d %d %d %d \n",
		b[0],b[1],b[2],b[3],b[4],b[5],b[6],b[7],b[8],b[9]);
	MyPrint(p->debugwin,buf, TRUE);

	MyPrint(p->debugwin,"Signed Integers \n", TRUE);
	engine->GetInts(ints);
	snprintf(buf,BUFFSIZE,"%li %li %li %li %li %li %li %li %li %li \n",
		ints[0],ints[1],ints[2],ints[3],ints[4],ints[5],ints[6],ints[7],ints[8],ints[9]);
	MyPrint(p->debugwin,buf, TRUE);

	MyPrint(p->debugwin,"Unsigned Integers \n", TRUE);
	engine->GetUInts(uints);
	snprintf(buf,BUFFSIZE,"%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu \n",
		uints[0],uints[1],uints[2],uints[3],uints[4],uints[5],uints[6],uints[7],uints[8],uints[9]);
	MyPrint(p->debugwin,buf, TRUE);

	MyPrint(p->debugwin,"Doubles \n", TRUE);
	engine->GetDbls(dbls);
	snprintf(buf,BUFFSIZE,"%.2f %.2f %.2f %.2f %.2f %.2f %.2f %.2f %.2f %.2f \n",
		dbls[0],dbls[1],dbls[2],dbls[3],dbls[4],dbls[5],dbls[6],dbls[7],dbls[8],dbls[9]);
	MyPrint(p->debugwin,buf, TRUE);

	MyPrint(p->debugwin,"CStrings \n", TRUE);
	engine->GetStrings(s);
	for (i=0;i<10;i++)
	{
		s[i].ConvertBinary();
		if (s[i].GetStringPtr()>0)
		{
			snprintf(buf,BUFFSIZE,"'%s' \n", s[i].GetStringPtr());
			MyPrint(p->debugwin,buf, TRUE);
			}
		else
		{
			snprintf(buf,BUFFSIZE,"'(NULL)' \n");
			MyPrint(p->debugwin,buf, TRUE);
			}
		}

	} ///////////////////////////////////////////////////////////////////////////////end Display

////////////////////////////////////////////////////////////////////////////////////////////////////
// ExecComm
////////////////////////////////////////////////////////////////////////////////////////////////////
void ExecComm(int menucontext, char c, curswin *p)
{
//char testmsg[]="This is a message from console. Hello World.\n";
//char msg[sizeof(int)];
//int  l,m;
int  result;
//bool timedout;
CString		myinput;

	switch (menucontext)
	{
	case MAIN_CONTEXT: 
		switch (c)
		{
		case 'r':
			// this will run the script file. 
			MyPrint(p->debugwin,"Run Script File \n", true);
			engine->Run();
			break;
			}

		break;

	case ASM_CONTEXT: 
		switch (c)
		{
		case 'o':
			{
			CString	*filename;
				// this will be a script file.	it will run in the script engine and 
				// will exercise the cxitem infrastructure.
				ThreadLog("ExecComm: Init the script engine Again.\n");	
				engine->Init();
				MyPrint(p->debugwin,"Open Script File \n", true);
				filename=GetAFileName(p);
				engine->Assemble(CE_ASM_OPEN,filename,cmdline);
				}
			break;

		case 'f':
			// this will run the first pass of the assembler. 
			MyPrint(p->debugwin,"Run First Pass \n", true);
			result=engine->Assemble(CE_ASM_FIRST,0,cmdline);
			if (result==CE_ERR_BADFILE)
			{		
				MyPrint(p->debugwin,"Assembly Error: Couldn't open file \n", true);
				}
			if (result==CE_ERR_NOINST)
			{		
				MyPrint(p->debugwin,"Assembly Error: Expected instruction \n", true);
				}
			if (result==CE_ERR_NOERR)
			{		
				MyPrint(p->debugwin,"First Pass Assembly Complete \n", true);
				}
			break;

		case 's':
			// this will run the second pass of the assembler. 
			MyPrint(p->debugwin,"Run Second Pass \n", true);
			result=engine->Assemble(CE_ASM_SECOND,0,cmdline);
			if (result==CE_ERR_NOERR)
			{		
				MyPrint(p->debugwin,"Second Pass Assembly Complete \n", true);
				}
			break;

		case 'l':
			// this will write the bin script. 
			MyPrint(p->debugwin,"Log bin log file\n", true);
			result=engine->Assemble(CE_ASM_LIST,0,cmdline);
			break;

		case 'w':
			// this will write the bin script. 
			MyPrint(p->debugwin,"Write Bin \n", true);
			break;
			}

		break;

	case FILE_CONTEXT: 
		switch (c)
		{
		case 'o':
			{
			CString	*filename;
			bool	result;
			CString Log;

				// this will be a script file.	it will run in the script engine and 
				// will exercise the cxitem infrastructure.
				MyPrint(p->debugwin,"Open Script (A Bin File) \n", true);
				filename=GetAFileName(p);
//				result=LoadFile(filename,p);
				result=LoadFile(filename,engine);
				if (!result)
				{
					Log="FAILED to open file for reading '"+*filename+"'\n"; MyPrint(p->debugwin, &Log, true);
					}
				else 
				{
					Log="Loaded Script File '"+*filename+"' OK\n"; MyPrint(p->debugwin, &Log, true);
//					snprintf(buf,BUFFSIZE,"Loaded Script File '%s' OK\n",filename->GetStringPtr());
//					MyPrint(p->debugwin,buf, TRUE);
					
					}
				
				}
			break;

		case 's':
			MyPrint(p->debugwin,"Save File \n", true);
			break;

		default:
			beep();                              //("Please try again\n");
			break;
			} // end FILE_CONTEXT
		break;

	case HELP_CONTEXT:
		switch (c)
		{
		case 'a':
			MyPrint(p->debugwin,"About \n", true);
			break;

		case 'h':

			MyPrint(p->debugwin,"Help \n", true);
			break;

		default:
			beep();                              //("Please try again\n");
			break;
			} // HELP_CONTEXT
		break;

	case TEST_CONTEXT:
		switch (c)
		{
		case 's':
			{
			int result;
		
				MyPrint(p->debugwin,"Single step \n", TRUE);
				result=engine->SingleStep();
				if (result!=CE_ERR_NOERR)
				{
				char    buf[BUFFSIZE];
					snprintf(buf,BUFFSIZE,"Engine error: %d\n",result);
					MyPrint(p->mainwin,buf, TRUE);
					}
				Display(p);
				}
			break;

		case 'r':
			MyPrint(p->debugwin,"Run \n", true);
			{
                        int     result;

				MyPrint(p->debugwin,"Run script \n", TRUE);
				result=engine->Run();
				if (result!=CE_ERR_NOERR)
				{
				char    buf[BUFFSIZE];
					snprintf(buf,BUFFSIZE,"Engine error: %d\n",result);
					MyPrint(p->mainwin,buf, TRUE);
					}
				}
			break;
		case 'd':
			MyPrint(p->debugwin,"Display \n", true);
			Display(p);
			break;

		case 'R':
			MyPrint(p->debugwin,"Reset \n", true);
			engine->Reset();
			break;

		case 'p':
			MyPrint(p->debugwin,"PC Set \n", true);
			{
			long int li;
				MyPrint(p->mainwin,"Enter PC  \n", TRUE);
				li=GetAnInt(p);
				engine->SetPC(li);
				}
			break;

		case 'l':
			MyPrint(p->debugwin,"List \n", true);
			{
			long int li;
				li=GetAnInt(p);
				ListScript(li,p);
				}
			break;

			}
		break; // End TEST_CONTEXT

//	break;

		}
	} //////////////////////////////////////////////////////////////////////////////end ExecComm

////////////////////////////////////////////////////////////////////////////////////////////////////
// CmdLineExe
////////////////////////////////////////////////////////////////////////////////////////////////////
int CmdLineExe()
{
bool		bresult;
int		result;
curswin		*winp;
CString		Log;

	// we arent going to use curses here so zero it out
	winp=0;	

	// execute a script and exit
	Log="CmdLineExe: Set CEngine StringLog \n"; Log.Log(); 
	Log="CmdLineExe: Set CEngine External Callbacks \n"; Log.Log(); 
	engine->Register(CEPrintInt,     0,1,0,0,0,0,0,0,0,0,1000,(void*)winp);
	engine->Register(CEPrintCString, 0,0,0,0,1,0,0,0,0,0,1001,(void*)winp);
	engine->Register(CEPrintfI,      0,1,0,0,0,0,0,0,0,0,1002,(void*)winp);
	engine->Register(CEPrintf,       0,0,0,0,1,0,0,0,0,0,1003,(void*)winp);
	engine->Register(CEPrintfU,      0,0,1,0,0,0,0,0,0,0,1004,(void*)winp);
	Log="CmdLineExe: Loadfile trying to load '"+exefile+"'\n"; Log.Log(); 
	bresult=LoadFile(&exefile,engine);
	if (bresult)
	{

		Log="CmdLineExe: Run Script \n"; Log.Log(); 
		result=engine->Run();
		if (result!=CE_ERR_NOERR)
		{
		char    buf[BUFFSIZE];
			Log="CmdLineExe: CEngine Error \n"; Log.Log(); 
			snprintf(buf,BUFFSIZE,"Engine error: %d\n",result);
			printf(buf);
			return(1);
			}
		}
	else	
	{
		Log="CmdLineExe: Loadfile failed \n"; Log.Log(); 
		return(1);
		}

	Log="CmdLineExe: CEngine OK \n"; Log.Log(); 
	return (0);
	} ////////////////////////////////////////////////////////////////////////////end CmdLineExe

////////////////////////////////////////////////////////////////////////////////////////////////////
// CmdLineAssemble
////////////////////////////////////////////////////////////////////////////////////////////////////
int CmdLineAssemble()
{
int	result;
CString L("NEW MESG First Pass Assembly Complete\n");

	// set filenames
//	printf("WIII Assembler Starting \n"); 
	engine->Assemble(CE_ASM_OPEN,&inputfilename,cmdline);
	outputfile.Log("Assem: output file is '","'\n");
	if (outputfile.Length()>0) 
	{
		engine->Assemble(CE_ASM_OUTPUT,&outputfile,cmdline);
		}

	if (lstfile.Length()>0) 
	{
		engine->Assemble(CE_ASM_LSTFILE,&lstfile,cmdline);
		}

//	printf("WIII Assembler Starting First Pass\n"); 
	// do assembly
	result=engine->Assemble(CE_ASM_FIRST,0,cmdline);
	if (result==CE_ERR_NOERR) 
	{
		printf("First Pass Assembly Complete \n"); 
		L.Log(); 
		result=engine->Assemble(CE_ASM_SECOND,0,cmdline);
	
		if (result==CE_ERR_NOERR) 
		{
			printf("Second Pass Assembly Complete \n"); 
			// write the output
			if (result==CE_ERR_NOERR)
			{
				result=engine->Assemble(CE_ASM_WRITE,0,cmdline);
				result=engine->Assemble(CE_ASM_LIST,0,cmdline);
				}
			}
		else
		{
			// Second pass failure
			printf("Second Pass Assembly Failed. \n"); 
			L="Second pass failed.\n";
			L.Log(); 
			}
		}
	else
	{
		// First pass failure
		printf("First Pass Assembly Failed. \n"); 
		L="First pass failed.\n";
		L.Log(); 
		}
	return (0);
	} ///////////////////////////////////////////////////////////////////////end CmdLineAssemble

////////////////////////////////////////////////////////////////////////////////////////////////////
// CmdLine
////////////////////////////////////////////////////////////////////////////////////////////////////
int CmdLine()
{
CString		StaticInit;
CString		Log;

	// do everything but talk to the user
	// first set up the script engine
	engine = new CEngine;
	engine->Init();
	engine->SetLogFunc(ThreadLog);
	engine->SetCStringLogFunc(ThreadLog);
	StaticInit.SetLogFunc(ThreadLog);


	if (assem) 
	{
		Log="CmdLine: Calling CmdLineAssemble\n"; Log.Log();
		CmdLineAssemble();
		}

	if (exe) 
	{
		Log="CmdLine: Calling CmdLineExe\n"; Log.Log();
		CmdLineExe();
		}

	return (0);
	} ///////////////////////////////////////////////////////////////////////////////end CmdLine

////////////////////////////////////////////////////////////////////////////////////////////////////
// Main
////////////////////////////////////////////////////////////////////////////////////////////////////
int main(int argc, char** argv)
{
char		choice;
//void            *param;
int		context, ret;
curswin		*winp;
CString		StaticInit;

	// init the screen & then threading and such
	// erase the screen
	Init(argc, argv);	// Init debug model
	StaticInit.SetLogFunc(ThreadLog);

	// Do they want to run on commandline ?
	if (cmdline) 
	{
		ret=CmdLine();
		exit(ret);
		}

	InitScreen(&winp);	// Init the curses library & screen mutex

	// Init the screen
	clear();
	refresh();

//	world = new CWorld;
	//list = new CXIterator;

	// init the script engine
	ThreadLog("Main: Init the script engine.\n");	
	engine = new CEngine;
	engine->Init();
	engine->SetLogFunc(ThreadLog);
	engine->SetCStringLogFunc(ThreadLog);
	engine->Register(CEPrintInt,     0,1,0,0,0,0,0,0,0,0,1000,(void*)winp);
	engine->Register(CEPrintCString, 0,0,0,0,1,0,0,0,0,0,1001,(void*)winp);
//	engine->Register(CEPrintf,       0,0,0,0,1,0,0,0,0,0,1002,(void*)winp);
//	engine->Register(CEPrintfI,      0,1,0,0,0,0,0,0,0,0,1003,(void*)winp);

	// Start the listener
	// pack any params for the listener thread
	// tlist is a global thread tracking structure
	//c[0]=' ';
	context=MAIN_CONTEXT;
	do  
	{
		choice=PrintMenu(&context,winp);	// this will update context as required
		ExecComm(context,choice,winp);
		} while (choice!='z');
	DeInit();
	MyPrint(winp->debugwin,"Before DeInitScreen \n", true);
	DeInitScreen(winp);
	printf("Returning from main.\n");
	} //////////////////////////////////////////////////////////////////////////////////end main

