////////////////////////////////////////////////////////////////////////////////////////////////////
// handler.cpp  
// This module contains routines so handle messages. Once the packets have been parsed and checked 
// they are enqueued into the CPool queue. Then each worked thread gets the next workchunk. Each 
// workchunk has a pointer to the correct cserver which contains the message in "cooked" format.
// Each thread calls into handler to handle each message. Handler is massively threaded. Extreme
// care must be take to ensure thread integrity.
//
// 
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci 
// Revised : 26 Jan 2006
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
// Includes
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <string.h>

//#include "wiii_types.h"
#include "wiii_structures.h"
#include "wiii_util.h"
#include "worldiii.h"
#include "cpool.hpp"
#include "cserver.hpp"

////////////////////////////////////////////////////////////////////////////////////////////////////
// Defines
////////////////////////////////////////////////////////////////////////////////////////////////////
//#define               BU FFSIZE        120

////////////////////////////////////////////////////////////////////////////////////////////////////
// Local Globals
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
// External Globals
////////////////////////////////////////////////////////////////////////////////////////////////////
extern	int		listenstate;
extern	bool            debug;                  // This puts server into debug mode 
extern  CUSock          *servsock;
extern  CIterator       serverlist;
extern	int		loglevel;
extern	void		ThreadLog(char * s);
extern	void		ConPrint(char *);
extern	void		ConPrint(CString *);

////////////////////////////////////////////////////////////////////////////////////////////////////
// Code
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
// ListPlayers
////////////////////////////////////////////////////////////////////////////////////////////////////
bool ListPlayers(CServer *cs)
{
char            buf[BUFFSIZE];
WIIHANDLE	curs;
char		*name;
bool		result;
	
	// log entry 
	if (loglevel>=1) ThreadLog("ListPlayers: Entered.");
	
	// get a private cursor from serverlist
	curs=serverlist.GetCursor((unsigned long)pthread_self());

	serverlist.MoveFirst(curs);
	while (!(serverlist.EndList(curs)))
	{
		if (loglevel>=3) ThreadLog("ListPlayers: Search loop.");

		// send message 
		name=((CServer*)serverlist.GetCurrent(curs))->GetPlayerName();
		snprintf(buf,BUFFSIZE,"ListPlayers: name='%s'.",name);
		ThreadLog(buf);
		result=((CServer*)cs)->SendToClient(name);
		if (!result)
		{
			serverlist.ReleaseCursor(curs,(unsigned long)pthread_self());
			return false;
			}

		// get next cs server
		serverlist.MoveNext(curs);
		}

	// release private cursor from serverlist and log exit
	serverlist.ReleaseCursor(curs,(unsigned long)pthread_self());
	if (loglevel>=1) ThreadLog("ListPlayers: Exited failed.");
	return true;

	} ///////////////////////////////////////////////////////////////////////////end ListPlayers

////////////////////////////////////////////////////////////////////////////////////////////////////
// SendTell
////////////////////////////////////////////////////////////////////////////////////////////////////
bool SendTell(CServer *cs)
{
char            buf[BUFFSIZE];
WIIHANDLE	curs;

	if (loglevel>=1) ThreadLog("SendTell: Entered.");

	// get a private cursor from serverlist
	curs=serverlist.GetCursor((unsigned long)pthread_self());

	// list all the servers for debug purposes
	serverlist.MoveFirst(curs);
	while (!(serverlist.EndList(curs)))
	{
		if (loglevel>=3) ThreadLog("SendTell: Search loop.");

		// look for the dest player
		if (   strcmp((((CServer*)(serverlist.GetCurrent(curs)))->GetCharacterName())
			,(cs->GetMessageParm(0)))==0)
		{
			// found dest character, so send the message
			if (((CServer*)(serverlist.GetCurrent(curs)))->SendToClient(cs->GetMessageParm(1))) 
			{
				serverlist.ReleaseCursor(curs,(unsigned long)pthread_self());
				return true;
				}
			else 
			{
				serverlist.ReleaseCursor(curs,(unsigned long)pthread_self());
				return false;
				}
			}
		// just in case we didnt go through the above if
		if (loglevel>=1) ThreadLog(buf);
		if (loglevel>=1) ConPrint(buf);

		// get next cs server
		serverlist.MoveNext(curs);
		}

	// release private cursor from serverlist
	serverlist.ReleaseCursor(curs,(unsigned long)pthread_self());
	if (loglevel>=1) ThreadLog("SendTell: Exited failed.");
	return false;
	} //////////////////////////////////////////////////////////////////////////////end SendTell

////////////////////////////////////////////////////////////////////////////////////////////////////
// SendToClient
////////////////////////////////////////////////////////////////////////////////////////////////////
void SendToClient(void *ptr)
{
	} //////////////////////////////////////////////////////////////////////////end SendToClient

////////////////////////////////////////////////////////////////////////////////////////////////////
// Handler
////////////////////////////////////////////////////////////////////////////////////////////////////
void Handler(void *ptr)
{
WORKCHUNK               *wc;
CServer                 *cs;
char			buf[BUFFSIZE];
CString			sbuf;

	// unpack our parm
	if (loglevel>=1) ThreadLog("Handler: Entered.");
	wc=(WORKCHUNK *)ptr;
	cs=wc->server;

	// For now just display is bad message
	if (cs->GetMessageErrorType()!=CMSG_NOERROR)
	{
		// show it and press on
		// tell everyone we got a bad message 
                snprintf(buf,WORLDMSGSIZE,"Handler: BAD MESSAGE [%d].  ",cs->GetMessageErrorType());
		cs->SendToClient(buf);
		ConPrint(buf);
		ThreadLog(buf);
		if (cs->DelMessage())
		{
			ThreadLog("Handler: Deleted bad message."); 
			}
		else
		{
			ThreadLog("Handler: ERROR: There was no message to delete.");
			}
		return;
		}

	// Handle messages
	if (loglevel>=1) ThreadLog("Handler: before switch.");
	switch (cs->GetMessage())
	{
	case WIIIM_PLAYERID:

		if (loglevel>=1) ThreadLog("Handler: WIIM_PLAYERID.");

		// Set client mode to char select, get charlist, send it to client
		snprintf(buf,BUFFSIZE,"Handler (PLAYERID): LOGIN NAME='%s' PASSWD='%s'.\n", 
				cs->GetMessageParm(0), 
				cs->GetMessageParm(1));
//		sbuf="Handler (PLAYERID): LOGIN NAME=";
//		sbuf=sbuf+(*(cs->GetMessageParmString(0)))+" PASSWD="+(*(cs->GetMessageParmString(1)))+".\n";
		ConPrint(buf);
		ThreadLog(buf);

		// send message to client
		cs->SendToClient(buf);
		cs->SetPlayerName();		//uses the contents of the message to set the name
		 
		break;

	case WIIIM_PLAY:

		// Set client mode to char select, get charlist, send it to client
		snprintf(buf,BUFFSIZE,"Handler (CHARACTER): NAME=%s .\n", 
				cs->GetMessageParm(0));
		ConPrint(buf);
		ThreadLog(buf);
		cs->SetCharacterName();		//uses the contents of the message to set the name
		cs->SendToClient(buf);

//		if (!CharSelect(m,s))
//		{
//			if (debug) ThreadLog("Handler: CharSelect Failed. ",s->GetServerLogName());
//			}
		break;

	case WIIIM_ACK:
		if (loglevel>=1) ThreadLog("Handler: WIIM_ACK.");
		break;

	case WIIIM_TELL:
		// send a message to another player
		if (loglevel>=1) ThreadLog("Handler: WIIM_TELL.");
		SendTell(cs);
		break;

	case WIIIM_LISTACTIVEPLAYERS:
		// the client wants to see all the logged in players
		if (loglevel>=1) ThreadLog("Handler: WIIM_LISTACTIVEPLAYERS.");
		snprintf(buf,BUFFSIZE,"Got listactive, sending players.\n");
		ConPrint(buf);
		ListPlayers(cs);
		break;

	case WIIIM_PING:
		if (loglevel>=1) ThreadLog("Handler: WIIM_PING.");
		// the client just sent me a ping
		// SO i send him an ACK
//		if (loglevel>=1) ConPrint("Got ping, sending ack.\n");
		snprintf(buf,BUFFSIZE,"Got ping, sending ack.\n");
		ConPrint(buf);
		snprintf(buf,BUFFSIZE,"%s", TOKEN_ACK);
		cs->SendToClient(buf);
		break;

        default:
                snprintf(buf,WORLDMSGSIZE,"Handler: UNHANDLED MESSAGE [%d].  ",cs->GetMessage());

		// tell everyone we got a bad message 
		cs->SendToClient(buf);
		ConPrint(buf);
		ThreadLog(buf);
		 
                break;
        	} // end switch

	// free the mem for the packet
	ThreadLog("Handler: Before cs->DelMessage().");
	if (cs->DelMessage())
	{
		ThreadLog("Handler: Normal message delete."); 
		}
	else
	{
		ThreadLog("Handler: ERROR: There was no message to delete.");
		}
	ThreadLog("Handler: Exited.");

	} ///////////////////////////////////////////////////////////////////////////////end Handler


