//////////////////////////////////////////////////////////////////////////////////////////////////////
// cxitem.cpp
//
// the purpose is to build a doubly linked list using pointers to objects
// then to be able to move a current pointer back and forth along the list
// we will also have the ability to implement a tree structure by having 
// parent and child pointers
// this is an extension of the other class. It supports multiple link pairs. THe reason for this
// is so we can have lists in different orders. EX sorting the list of objects by x or y or z
// coordinates. Also supports having sub lists.
//
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci 
// Revised : 01 Jun 2007
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
//////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////
// Some notes about this data structure
//
// One thing to remember! The CIterator *is* the data structure. Inside the Citerator are the sublists
// and the cursors that work in the assigned sublists.
//
// Things you can assume
// - That a list starts and ends at pointers[list].head and pointers[list].tail
// - That if a list starts at pointers[i].head it uses the 'i'th entry for links throughout.
// - That unused list entries in the head and tail arrays and in CXItems are set to 0
// - when an ITEM is deleted all lists that item is in are properly updated
// - an item is likely NOT at the same place in each list. IE an item at the head of one list may well
//   be in the middle of another, and may not even be in a third list.
//
// Things you can not assume
// - every item need not be in every list
// - when a list is deleted no check is made to ensure that items are still reachable by another list
// - cursor may not always be pointing at a valid object, (ie at end of list, see move next)
// PROBLEM !! What if a thread deletes an object that is pointed to by another cursor ? 
//////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////
// Includes
//////////////////////////////////////////////////////////////////////////////////////////////////////
#include <curses.h>

#include "cxitem.hpp"	   
#include "worldiii.h"
#include "wiii_util.h"

////////////////////////////////////////////////////////////////////////////////////////////////////
// Externals
////////////////////////////////////////////////////////////////////////////////////////////////////
extern	int		loglevel;
extern	void		ThreadLog(char *s);

////////////////////////////////////////////////////////////////////////////////////////////////////
// Globals
////////////////////////////////////////////////////////////////////////////////////////////////////
int CXItem::itemcount=0;            //init the static member
int CXItem::itemid=0;            //init the static member

////////////////////////////////////////////////////////////////////////////////////////////////////
//CXNode
////////////////////////////////////////////////////////////////////////////////////////////////////
CXNode::CXNode()
{
     child=0;
     nextnode=0;
     }

////////////////////////////////////////////////////////////////////////////////////////////////////
//Node Destructor
////////////////////////////////////////////////////////////////////////////////////////////////////
CXNode::~CXNode()
{
     }

////////////////////////////////////////////////////////////////////////////////////////////////////
//Default Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////
CXItem::CXItem()
{
int	i;

//	InitMutex(&rwlock);
	InitRWLock(&rwlock);

	id=itemid++;             //set the id upon creation
	itemcount++;
	value=0;
	for (i=0;i<MAXLISTS;i++)
	{
		// zero out the lists arrays
		nextitem[i]=0;
		previtem[i]=0;
		}

//	childlist=0;            //points to the child list
	parent=0;               //points to child 
	errorstate=ERR_NOERROR;
	delflag=false;
	} ///////////////////////////////////////////////////////////////////////end ItemConstructor

////////////////////////////////////////////////////////////////////////////////////////////////////
//Default Destructor
////////////////////////////////////////////////////////////////////////////////////////////////////
CXItem::~CXItem()
{
	itemcount--;  //decrease the item count
	} ////////////////////////////////////////////////////////////////////////end ItemDestructor


////////////////////////////////////////////////////////////////////////////////////////////////////
//Default Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////
CXIterator::CXIterator()
{
int		i;
int		templist;
WIIIHANDLE	tempcurs;

	InitMutex(&mutex);
	cptr=0;
	errorstate=ERR_NOERROR;
	state=CX_STATE_NOTINIT;
	maxlistcount=MAXLISTS;

	// Init cursors
	for (i=0;i<MAXCURSORS;i++)
	{
		cursarray[i].curr=0;			// Not pointing to a CXItem right now
		cursarray[i].inuse=false;		// Not in use right now
		cursarray[i].list=-1;			// 
		cursarray[i].serverid=INVALIDSERVER;	// No real server has this guy right now
							// depend on "inuse" as INVALIDSERVER may be a valid thread
							// we're really just saying that the curs is not is use
		}

	for (i=0;i<maxlistcount;i++)
	{
		pointers[i].head=0;
		pointers[i].tail=0;
		pointers[i].inuse=false;
		InitMutex(&(pointers[i].mutex));
		}

	// first list [0], and first cursor [0] will always be in use. 
	// it will be used to track ALL items.
	// any append or insert will automatically add to list 0
	// These use the locks so should be after the InitMutex
	AddLinkedList(&templist);		// this is to create the first [0] list
	GetCursor(templist, 0, &tempcurs);	// this is to create the first [0] cursor

	} ///////////////////////////////////////////////////////////////////end IteratorConstructor

////////////////////////////////////////////////////////////////////////////////////////////////////
//Default Destructor
//- start at one end, if it is a child then orphan it
//- then delete it
//- a child can only have one parent so, when it is 
//  orphaned then we can delete it
////////////////////////////////////////////////////////////////////////////////////////////////////
CXIterator::~CXIterator()
{
int	i;
CXItem	*p;

	// have to do it the hard way
	for (i=0; i<maxlistcount; i++)
	{
		// search each list
		if (pointers[i].inuse)
		{
			// this array is in use so delete
			while (p=pointers[i].head)
			{
				pointers[i].head=pointers[i].head->nextitem[i];
				delete p;
				}
			}
		}
	} ////////////////////////////////////////////////////////////////////end IteratorDestructor

////////////////////////////////////////////////////////////////////////////////////////////////////
// ItemReadLock
////////////////////////////////////////////////////////////////////////////////////////////////////
int CXItem::ItemReadLock()
{
	// lock the rwlock for read only
	return(MyReadLock(&rwlock,true,"CXIterator::ItemReadLock"));
	} //////////////////////////////////////////////////////////////////////////end ItemReadLock

////////////////////////////////////////////////////////////////////////////////////////////////////
// ItemWriteLock
////////////////////////////////////////////////////////////////////////////////////////////////////
int CXItem::ItemWriteLock()
{
	// lock the rwlock
	return(MyWriteLock(&rwlock,true,"CXIterator::ItemWriteLock"));
	} /////////////////////////////////////////////////////////////////////////end ItemWriteLock

////////////////////////////////////////////////////////////////////////////////////////////////////
// ItemTryWriteLock
////////////////////////////////////////////////////////////////////////////////////////////////////
int CXItem::ItemTryWriteLock()
{
	// Try to lock the rwlock
	return(MyTryWriteLock(&rwlock,true,"CXIterator::ItemWriteLock"));
	} //////////////////////////////////////////////////////////////////////end ItemTryWriteLock

////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
// ItemUnLock
////////////////////////////////////////////////////////////////////////////////////////////////////
int CXItem::ItemUnLock()
{
	// unlock the mutex
	return(MyRWUnLock(&rwlock,true,"CXIterator::ItemUnLock"));
	} ////////////////////////////////////////////////////////////////////////////end ItemUnLock

////////////////////////////////////////////////////////////////////////////////////////////////////
// ListLock
////////////////////////////////////////////////////////////////////////////////////////////////////
int CXIterator::ListLock(WIIILIST list)
{
	// lock the mutex
	return(MyLock(&(pointers[list].mutex),true,"CXIterator::ListLock"));
	} //////////////////////////////////////////////////////////////////////////////end ListLock

////////////////////////////////////////////////////////////////////////////////////////////////////
// ListUnLock
////////////////////////////////////////////////////////////////////////////////////////////////////
int CXIterator::ListUnLock(WIIILIST list)
{
	// unlock the mutex
	return(MyUnLock(&(pointers[list].mutex),true,"CXIterator::ListUnLock"));
	} ////////////////////////////////////////////////////////////////////////////end ListUnLock

////////////////////////////////////////////////////////////////////////////////////////////////////
// IterLock
////////////////////////////////////////////////////////////////////////////////////////////////////
int CXIterator::IterLock()
{
	// lock the mutex
	return(MyLock(&mutex,true,"CXIterator::Lock"));
	} //////////////////////////////////////////////////////////////////////////////end IterLock

////////////////////////////////////////////////////////////////////////////////////////////////////
// IterUnLock
////////////////////////////////////////////////////////////////////////////////////////////////////
int CXIterator::IterUnLock()
{
	// unlock the mutex
	return(MyUnLock(&mutex,true,"CXIterator::UnLock"));
	} ////////////////////////////////////////////////////////////////////////////end IterUnLock

////////////////////////////////////////////////////////////////////////////////////////////////////
// CheckCursor
// Private
// Checking against 1 is to implement the zero list for delete only use
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CXIterator::CheckCursor(WIIIHANDLE curs)
{

	// if curs out of bounds
	if (curs<1 || curs>=MAXCURSORS)  
	{
		printf("CXIterator::CheckCursor: Failed cursor range test\n");
		return(false); 
		}
	
	// check to ensure cursor in use
	if (!cursarray[curs].inuse)  
	{
		printf("CXIterator::CheckCursor: Failed inuse test \n");
		return(false); 
		}

	return (true);
	} ///////////////////////////////////////////////////////////////////////////end CheckCursor

////////////////////////////////////////////////////////////////////////////////////////////////////
// P_Appends something to the list at the tail end
// The zero list and the zero cursor eist only to ensure we can delete everything at the end.
// They cannot be directly referenced. They are only implicitly referenced. You cannot add something
// only to the zero list.
////////////////////////////////////////////////////////////////////////////////////////////////////
void CXIterator::P_AppendItem(WIIIHANDLE curs, CXItem *additem)
{    
WIIILIST	list;

	// make a convenient temp var
	list=cursarray[curs].list;

	// do the empty list case
	if ((pointers[list].head==0)&&(pointers[list].tail==0)) 
	{
		pointers[list].head=pointers[list].tail=additem;
		}
	else  //non empty list case
	{
		pointers[list].tail->nextitem[list]=additem;
		additem->previtem[list]=pointers[list].tail;
		pointers[list].tail=additem;
		}

	return;
	} //////////////////////////////////////////////////////////////////////////end P_AppendItem

////////////////////////////////////////////////////////////////////////////////////////////////////
// Appends something to the list at the tail end
// The zero list and the zero cursor exist only to ensure we can delete everything at the end.
// They cannot be directly referenced. They are only implicitly referenced. You cannot add something
// only to the zero list.
////////////////////////////////////////////////////////////////////////////////////////////////////
CXItem *CXIterator::AppendItem(WIIIHANDLE curs, CXItem *additem)
{    
WIIILIST	list;

	// Check for valid cursor
	if (!CheckCursor(curs)) { return 0; }

	ListLock(list); 

	// Check for valid cursor
	if (!CheckCursor(curs)) { ListUnLock(list); return(0); }

	// do the zero list case
	P_AppendItem(0,additem);

	// do the requested list case
	P_AppendItem(curs,additem);

	ListUnLock(list);
	return (cursarray[curs].curr);
	} ////////////////////////////////////////////////////////////////////////////end AppendItem

////////////////////////////////////////////////////////////////////////////////////////////////////
// displays the whole list
////////////////////////////////////////////////////////////////////////////////////////////////////
void CXIterator::DisplayList(WIIIHANDLE curs)
{
CXItem *temp;
int i;
WIIILIST	list;

	// Check for valid cursor
	if (!CheckCursor(curs)) { return; }

	i=0;
	list=cursarray[curs].list;
	if ((pointers[list].head==0) || (pointers[list].tail==0)) return;
	temp=GetCurrent(curs);
	MoveFirst(curs);		//this shows how to use member
	while (!EndList(curs))		//functions inside other member
	{				//functions to accomplish work
		MoveNext(curs);		//using member functions is                                            
		}			//probably better because i can
	SetCurrent(curs, temp);
	} ///////////////////////////////////////////////////////////////////////////end DisplayList

////////////////////////////////////////////////////////////////////////////////////////////////////
//displays the whole list backwards 
//starting from the tail item
////////////////////////////////////////////////////////////////////////////////////////////////////
void CXIterator::BackDisplayList(WIIIHANDLE curs)
{
int i;
WIIILIST	list;
 
	// Check for valid cursor
	if (!CheckCursor(curs)) { return; }

	i=0;
	list=cursarray[curs].list;
	if ((pointers[list].head==0) || (pointers[list].tail==0)) return;
	//this is another way to do the while loop to print out the list
	//by actually accessing the member variables directly

	} ///////////////////////////////////////////////////////////////////////end BackDisplayList
          
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Move prev returns a pointer to the prev item
// Provides external thread protection for a private routine that actually moves the pointers
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
CXItem *CXIterator::MovePrev(WIIIHANDLE curs)
{
CXItem		*retval;
WIIILIST	list;

        // First provide external thread protection
        // Lock The top level iterator.
        IterLock();

        // Check cursor
        if (!CheckCursor(curs)) {IterUnLock(); return(0);}

        // Make it easier to read
        list=cursarray[curs].list;

        if (P_StdChecks(curs,list))
        {
                // Grab the list for exclusive
                ListLock(list);

                // Let others back into the Iterator
                IterUnLock();

                // Second 
                // do the move
                retval=P_MovePrev(curs,list);

                ListUnLock(list);
                return (retval);
                }
        else
        {
                IterUnLock();
                return(0);
                }

//	// make things a little easier to read
//	list=cursarray[curs].list;
//
//	ListLock(list);
//
//	// Valid cursor
//	if (!CheckCursor(curs)) { ListUnLock(list); return(0); }
//
//	// handle the empty list case
//	if ((pointers[list].head==0) && (pointers[list].tail==0)) 
//	{
//		ListUnLock(list);
//		return(0);
//		}
//	
//	// handle the case of non valid curr
//	if (cursarray[curs].curr==0)
//	{
//		ListUnLock(list);
//		return (0);
//		}
//
//	// case of pointing at head of list
//	if (cursarray[curs].curr==pointers[list].head) 
//	{
//		cursarray[curs].curr=0;			// it doesnt matter which way we are going.
//							// after the last item in list. curr=0
//		ListUnLock(list);
//		return (cursarray[curs].curr);
//		}
//
//	// Handle the general case
//	cursarray[curs].curr=cursarray[curs].curr->previtem[list];
//
//
//	ListUnLock(list);
//	return (cursarray[curs].curr);           //return what it was
	} //////////////////////////////////////////////////////////////////////////////end MovePrev

////////////////////////////////////////////////////////////////////////////////////////////////////
// P_InsertItem
// A general insert statement that inserts before the item where next is pointing, 
// returns a pointer to the item inserted. 
// If there is an error (invalid cursor) return 0 and leave pointers unchanged.
//
// The zero list and the zero cursor exist only to ensure we can delete everything at the end.
// They cannot be directly referenced. They are only implicitly referenced. You cannot add something
// only to the zero list.
////////////////////////////////////////////////////////////////////////////////////////////////////
void CXIterator::P_InsertItem(WIIIHANDLE curs, CXItem *additem)
{
WIIILIST	list;

	// make things a little easier to read
	list=cursarray[curs].list;

	// handle the empty list case
	if ((pointers[list].head==0) && (pointers[list].tail==0)) 
	{
		ThreadLog("CXIterator:: InsertItem nothing in list");
		pointers[list].head=pointers[list].tail=additem;
		cursarray[curs].curr=additem;
		return;
		}

	// handle single item in list
	if ( pointers[list].head==pointers[list].tail ) 
	{
		ThreadLog("CXIterator:: InsertItem Single");
		additem->nextitem[list]=pointers[list].head;
		pointers[list].head->previtem[list]=additem;
		pointers[list].head=additem;
		cursarray[curs].curr=additem;
		return;
		}

	// handle case of if curr is at first in list 
	if (pointers[list].head==cursarray[curs].curr)
	{
		ThreadLog("CXIterator:: InsertItem pointing to first in list");
		additem->nextitem[list]=pointers[list].head;
		pointers[list].head->previtem[list]=additem;
		pointers[list].head=additem;
		cursarray[curs].curr=additem;
		return;
		}
	else    // this is the insert into multiple item list not at first position
	{
		if (cursarray[curs].curr==0) // 20000227: at least two items in list. 
		{
			// we probably just walked the list
			// so make curr=tail
			ThreadLog("CXIterator:: InsertItem setting curr=tail");
			cursarray[curs].curr=pointers[list].tail;
			}

		additem->nextitem[list]=cursarray[curs].curr;
		cursarray[curs].curr->previtem[list]->nextitem[list]=additem;
		additem->previtem[list]=additem->nextitem[list]->previtem[list];
		cursarray[curs].curr->previtem[list]=additem;
		cursarray[curs].curr=additem;
		return;
		}

	} //////////////////////////////////////////////////////////////////////////end P_InsertItem

////////////////////////////////////////////////////////////////////////////////////////////////////
// Insert item
// A general insert statement that inserts before the item where next is pointing, 
// returns a pointer to the item inserted. 
// If there is an error (invalid cursor) return 0 and leave pointers unchanged.
//
// The zero list and the zero cursor exist only to ensure we can delete everything at the end.
// They cannot be directly referenced. They are only implicitly referenced. You cannot add something
// only to the zero list.
////////////////////////////////////////////////////////////////////////////////////////////////////
CXItem *CXIterator::InsertItem(WIIIHANDLE curs, CXItem *additem)
{
WIIILIST	list;


	IterLock();
	
	// Make it easier to read
	list=cursarray[curs].list;

	ListLock(list); 

	IterUnLock();

	// Check for valid cursor
	if (!CheckCursor(curs)) { ListUnLock(list); return(0); }

	// Add to zero list
	P_InsertItem(0, additem);

	// Add to selected list
	P_InsertItem(curs, additem);

	ListUnLock(list);
	return (cursarray[curs].curr);   
	} ////////////////////////////////////////////////////////////////////////////end InsertItem

////////////////////////////////////////////////////////////////////////////////////////////////////
//returns a pointer if the item is in 
//the list, curr unaffected     
////////////////////////////////////////////////////////////////////////////////////////////////////
CXItem *CXIterator::FindItem(WIIIHANDLE curs, CXItem *r)
{
CXItem *p;
WIIILIST	list;

	// Make it easier to read
	list=cursarray[curs].list;

	ListLock(list);

	// Check for valid cursor
	if (!CheckCursor(curs)) { ListUnLock(list); return(0); }

	if (pointers[cursarray[curs].list].head==0) { ListUnLock(list); return(0);}
	p=pointers[cursarray[curs].list].head;
	while (p!=0)  //////////////////////////////////check this!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	{
		if (p==r) { ListUnLock(list); return (r);} //found the record
		p=p->nextitem[cursarray[curs].list];
		}
	ListUnLock(list);
	return (0); //did not find the record we were looking for
	} //////////////////////////////////////////////////////////////////////////////end FindItem

////////////////////////////////////////////////////////////////////////////////////////////////////
// SetCurrent
// this uses finditem to ensure that 
// the item is in the list
// then sets the pointer
////////////////////////////////////////////////////////////////////////////////////////////////////
CXItem *CXIterator::SetCurrent(WIIIHANDLE curs, CXItem *r)
{
CXItem *p;
WIIILIST	list;

	// Make it easier to read
	list=cursarray[curs].list;

	ListLock(list);

	// Check for valid cursor
	if (!CheckCursor(curs)) { ListUnLock(list); return(0); }

	if ((p=FindItem(curs,r))) 
	{
		cursarray[curs].curr=p;
		ListUnLock(list);
		return (cursarray[curs].curr);
		}
	else {ListUnLock(list); return(0);}        
	} ////////////////////////////////////////////////////////////////////////////end SetCurrent

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetCurrent
// FIXME should GetCurrent Lock the item ?
////////////////////////////////////////////////////////////////////////////////////////////////////
CXItem *CXIterator::GetCurrent(WIIIHANDLE curs) 
{
	// Check for valid cursor
//	if (!CheckCursor(curs)) { UnLock(); return(0); }
	if (!CheckCursor(curs)) { return(0); }

	return (cursarray[curs].curr);
	} ////////////////////////////////////////////////////////////////////////////end GetCurrent

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetCursor
// This is going to be called by many threads so we must protect the object from concurrent access
// Watch for deadlocks
// These cursors allow for many threads to have their own "current" position in the list maintained
// by this iterator
////////////////////////////////////////////////////////////////////////////////////////////////////
int CXIterator::GetCursor(WIIILIST l, unsigned long s, WIIIHANDLE *curs)
{
int	i;

	if (loglevel>=4) ThreadLog("CXIterator:: GetCursor: Entered ");
				
	IterLock();
	if (!pointers[l].inuse) {IterUnLock(); return ERR_LIST_NOTINUSE;}
	for (i=0;i<MAXCURSORS;i++)
	{
		if (cursarray[i].inuse==false) goto GetCursor0;
		}
	IterUnLock();
	if (loglevel>=4) ThreadLog("CXIterator:: GetCursor: Exit no cursors left");
	return (ERR_INVALID_CURSOR);

GetCursor0:
	
	// Setup cursor
	cursarray[i].inuse=true;
	cursarray[i].serverid=s;
	cursarray[i].curr=pointers[l].head;
	cursarray[i].list=l;
	state=CX_STATE_CURS;

	// record how many cursors we have in use now. Why ? Because we can ?
	curscount++;

	// Get Ready to return it.
	*curs=i;
	IterUnLock();			// once i set the flag true its mine
	if (loglevel>=4) ThreadLog("CXIterator:: GetCursor: Exit ok");
	return(ERR_NOERROR);
	} /////////////////////////////////////////////////////////////////////////////end GetCursor

////////////////////////////////////////////////////////////////////////////////////////////////////
// ReleaseCursor
////////////////////////////////////////////////////////////////////////////////////////////////////
int CXIterator::ReleaseCursor(WIIIHANDLE curs, unsigned long serverid)
{

	// ok now release it
	IterLock();
	if (!pointers[cursarray[curs].list].inuse) {IterUnLock(); return ERR_LIST_NOTINUSE;}
	if (cursarray[curs].serverid!=serverid) {IterUnLock(); return ERR_INVALIDSERVER;}
	if ((curs>=MAXCURSORS)||(curs<0)) {IterUnLock(); return ERR_INVALID_CURSOR;}
	if (cursarray[curs].inuse==true)
	{
	int	i;

		// release it
		cursarray[curs].inuse=false;
		cursarray[curs].serverid=INVALIDSERVER;
		cursarray[curs].curr=0;
		cursarray[curs].list=-1;

		// Keep track of howmany is use
		curscount--;

		// now check to see if any cursors still exist
		state=CX_STATE_LIST;
		for (i=0;i<MAXCURSORS;i++) if (cursarray[curs].inuse) state=CX_STATE_CURS;

		IterUnLock();
		return (true);
		}
	else
	{
		IterUnLock();
		return(ERR_CURSOR_NOTINUSE);
		}
	} /////////////////////////////////////////////////////////////////////////end ReleaseCursor

////////////////////////////////////////////////////////////////////////////////////////////////////
// MoveLast
////////////////////////////////////////////////////////////////////////////////////////////////////
CXItem *CXIterator::MoveLast(WIIIHANDLE curs)
{
CXItem 		*retval;
WIIILIST	list;

	// First provide external thread protection
	// Lock The top level iterator.
	IterLock();

	// Check cursor
	if (!CheckCursor(curs)) {IterUnLock(); return(0);}

	// Make it easier to read
	list=cursarray[curs].list;

        if (P_StdChecks(curs,list))
        {
		// Grab the list for exclusive
		ListLock(list);

		// Let others back into the Iterator
		IterUnLock();

                // Second
                // do the move
                retval=P_MoveLast(curs,list);

                ListUnLock(list);
                return (retval);
		}
	else
	{
                IterUnLock();
                return(0);
		}
	
	} //////////////////////////////////////////////////////////////////////////////end MoveLast

////////////////////////////////////////////////////////////////////////////////////////////////////
// MoveFirst
////////////////////////////////////////////////////////////////////////////////////////////////////
CXItem *CXIterator::MoveFirst(WIIIHANDLE curs)
{
CXItem 		*retval;
WIIILIST	list;

	// First provide external thread protection
	// Lock The top level iterator.
	IterLock();

	// Check cursor
	if (!CheckCursor(curs)) {IterUnLock(); return(0);}

	// Make it easier to read
	list=cursarray[curs].list;

        if (P_StdChecks(curs,list))
        {
		// Grab the list for exclusive
		ListLock(list);

		// Let others back into the Iterator
		IterUnLock();

                // Second
                // do the move
                retval=P_MoveFirst(curs,list);

                ListUnLock(list);
                return (retval);
		}
	else
	{
                IterUnLock();
                return(0);
		}
	
	} /////////////////////////////////////////////////////////////////////////////end MoveFirst

////////////////////////////////////////////////////////////////////////////////////////////////////
// MoveLast
////////////////////////////////////////////////////////////////////////////////////////////////////
//CXItem *CXIterator::MoveLast(WIIIHANDLE curs)
//{
//WIIILIST	list;
//
//	// Make it easier to read
//	list=cursarray[curs].list;
//
//	ListLock(list);
//
//	// Check for valid cursor
//	if (!CheckCursor(curs)) { ListUnLock(list); return(0); }
//
//	// Move
//	cursarray[curs].curr=pointers[cursarray[curs].list].tail;
//
//	ListUnLock(list);
//	return (cursarray[curs].curr);
//	} /////////////////////////////////////////////////////////////////////////////end MoveLast

////////////////////////////////////////////////////////////////////////////////////////////////////
// P_StdChecks
// Check some basic stuff return true if ok false if not
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CXIterator::P_StdChecks(WIIIHANDLE curs, WIIILIST list)
{
	// Valid cursor,  dont use bad list number, handle the empty list case, handle the null curs case
	if (!CheckCursor(curs)) 					return(false);
	if ((list>=maxlistcount)||(list<=0))  				return(false);
	if ((pointers[list].head==0) || (pointers[list].tail==0))	return(false);
	if (cursarray[curs].curr==0) 					return(false);
	return true;
	} ///////////////////////////////////////////////////////////////////////////end P_StdChecks
 
////////////////////////////////////////////////////////////////////////////////////////////////////
// P_MoveLast 
// Returns a pointer to the last item
// and moves the internal pointer to the last item
// Moves towards tail
////////////////////////////////////////////////////////////////////////////////////////////////////
CXItem *CXIterator::P_MoveLast(WIIIHANDLE curs, WIIILIST list)
{

	// Handle pointing at first item
	cursarray[curs].curr==pointers[list].tail;                       
	
	// handle general case
	while (cursarray[curs].curr!=0)
	{
		if (!((cursarray[curs].curr)->delflag))	return cursarray[curs].curr;
	
		// skip any marked delete records
		cursarray[curs].curr=(cursarray[curs].curr)->previtem[list];
		}
	return(0);

	} ////////////////////////////////////////////////////////////////////////////end P_MoveLast

////////////////////////////////////////////////////////////////////////////////////////////////////
// P_MoveFirst 
// Returns a pointer to the first item
// and moves the internal pointer to the first item
// Moves towards head
////////////////////////////////////////////////////////////////////////////////////////////////////
CXItem *CXIterator::P_MoveFirst(WIIIHANDLE curs, WIIILIST list)
{

	// Handle pointing at first item
	cursarray[curs].curr==pointers[list].head;                       
	
	// handle general case
	while (cursarray[curs].curr!=0)
	{
		if (!((cursarray[curs].curr)->delflag))	
		{
			return cursarray[curs].curr;
			printf("CXIterator::P_MoveFirst: returned item.  \n");
			}
	
		// skip any marked delete records
		cursarray[curs].curr=(cursarray[curs].curr)->nextitem[list];
		printf("CXIterator::P_MoveFirst: skipped deleted  \n");
		}
	printf("CXIterator::P_MoveFirst: returning NULL \n");
	return(0);

	} ///////////////////////////////////////////////////////////////////////////end P_MoveFirst

////////////////////////////////////////////////////////////////////////////////////////////////////
// P_MovePrev 
// Returns a pointer to the prev item
// and moves the internal pointer to the prev item
// Moves towards head
////////////////////////////////////////////////////////////////////////////////////////////////////
CXItem *CXIterator::P_MovePrev(WIIIHANDLE curs, WIIILIST list)
{
	// Handle pointing at last item
	if (cursarray[curs].curr==pointers[list].head)
	{
		cursarray[curs].curr=0;
		return (0);             //we should stick to convention, and return the nill pointer,
		}                       //this will make the lists work the way we unconsciously expect them to

	// move prev
	cursarray[curs].curr=(cursarray[curs].curr)->previtem[list];

	// handle general case
	while (cursarray[curs].curr!=0)
	{
		if (!((cursarray[curs].curr)->delflag)) return cursarray[curs].curr;
		
		// skip any marked delete records
		cursarray[curs].curr=(cursarray[curs].curr)->previtem[list];
		}
	return(0);
	} ////////////////////////////////////////////////////////////////////////////end P_MovePrev

////////////////////////////////////////////////////////////////////////////////////////////////////
// P_MoveNext 
// Returns a pointer to the next item
// and moves the internal pointer to the next item
// Moves towards tail
// On entry cursarray[curs]!=0
////////////////////////////////////////////////////////////////////////////////////////////////////
CXItem *CXIterator::P_MoveNext(WIIIHANDLE curs, WIIILIST list)
{

	// Handle pointing at last item
	if (cursarray[curs].curr==pointers[list].tail)
	{
		cursarray[curs].curr=0;
		return (0);             //we should stick to convention, and return the nill pointer,
		}                       //this will make the lists work the way we unconsciously expect them to

	// move next
	cursarray[curs].curr=(cursarray[curs].curr)->nextitem[list];

	// handle general case
	while (cursarray[curs].curr!=0)
	{
		if (!((cursarray[curs].curr)->delflag)) return cursarray[curs].curr;
		
		// skip any marked delete records
		cursarray[curs].curr=(cursarray[curs].curr)->nextitem[list];
		}
	return(0);
	} ////////////////////////////////////////////////////////////////////////////end P_MoveNext

////////////////////////////////////////////////////////////////////////////////////////////////////
// MoveNext 
// Provides external thread protection for a private routine that actually moves the pointers
////////////////////////////////////////////////////////////////////////////////////////////////////
CXItem *CXIterator::MoveNext(WIIIHANDLE curs)
{
CXItem *retval;
WIIILIST	list;

	// First provide external thread protection
	// Lock The top level iterator.
	IterLock();

	// Check cursor
	if (!CheckCursor(curs)) {IterUnLock(); return(0);}

	// Make it easier to read
	list=cursarray[curs].list;

	if (P_StdChecks(curs,list))
	{
		// Grab the list for exclusive
		ListLock(list);
	
		// Let others back into the Iterator
		IterUnLock();

		// Second 
		// do the move
		retval=P_MoveNext(curs,list);

		ListUnLock(list);
		return (retval);
		}
	else 
	{
		IterUnLock();
		return(0);
		}

	} //////////////////////////////////////////////////////////////////////////////end MoveNext

////////////////////////////////////////////////////////////////////////////////////////////////////
// BegList
// Are we pointing at the first item in list ?
// Use MoreItems() to test end of processing the ENTIRE list
// This gives us same behavior in both directions.
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CXIterator::BegList(WIIIHANDLE curs)
{
bool f;
WIIILIST	list;	

	// get list FIXME should this be in the lock too ?
	list=cursarray[curs].list;

	ListLock(list);	

	// Valid cursor
	// Return false for inv cursor to indicate NOT at a valid BegList
	if (!CheckCursor(curs)) { ListUnLock(list); return(false); }

	// Check
	if ((cursarray[curs].curr)==(pointers[cursarray[curs].list].head)) f=true; else f=false;

	ListUnLock(list);
	return (f);
	} ///////////////////////////////////////////////////////////////////////////////end BegList

////////////////////////////////////////////////////////////////////////////////////////////////////
// EndList
// Are we pointing at the LAST item in list ?
// Use MoreItems() to test end of processing the ENTIRE list
// This gives us same behavior in both directions.
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CXIterator::EndList(WIIIHANDLE curs) 
{	
bool f;
WIIILIST	list;	

	// get list FIXME should this be in the lock too ?
	list=cursarray[curs].list;

	ListLock(list);	

	// Valid cursor
	// Return false for inv cursor to indicate NOT at a valid EndList
	if (!CheckCursor(curs)) { ListUnLock(list); return(false); }

	// Check
	if ((cursarray[curs].curr)==(pointers[cursarray[curs].list].tail)) f=true; else f=false;

	ListUnLock(list);
	return (f);
	} ///////////////////////////////////////////////////////////////////////////////end EndList

////////////////////////////////////////////////////////////////////////////////////////////////////
// MoreItems
// Are we pointing at a Valid item in list ?
// Use MoreItems() to test end of processing the ENTIRE list
// This gives us same behavior in both directions.
// Since threads should not share cursors, a check of this after a move will yield valid results.
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CXIterator::MoreItems(WIIIHANDLE curs) 
{	
bool f;
WIIILIST	list;	

	// get list FIXME should this be in the lock too ?
	list=cursarray[curs].list;

	ListLock(list);

	// Valid cursor
	// Return false for inv cursor to indicate no more items
	if (!CheckCursor(curs)) { ListUnLock(list); return(false); }

	if ((cursarray[curs].curr)==0) f=false; else f=true;
	ListUnLock(list);
	return(f);
	} /////////////////////end MoreItems

////////////////////////////////////////////////////////////////////////////////////////////////////
// AddLinkedList
// This sets up the next linked list system with
//	- ad head[i] and tail[i] as the head and tail
//	- and with the nextitem[i] and previtem[i]
// note that each head[i] and tail[i] will only use their respective nextitem[i] and previtem[i]
// ie head[2] and tail[2] will only use nextitem[2] and previtem[2] on each object
// also each cursor will be for a particular i. 
////////////////////////////////////////////////////////////////////////////////////////////////////
int CXIterator::AddLinkedList(int *list)
{
int	i;

	if (loglevel>=4) ThreadLog("CXIterator:: AddLinkedList: Entered ");
				
	IterLock();
	for (i=0;i<maxlistcount;i++)
	{
		if (pointers[i].inuse==false) goto AddLinkedList0;
		}
	IterUnLock();
	if (loglevel>=4) ThreadLog("CXIterator:: AddLinkedList: Exit no cursors left");
	return (ERR_TOOMANYLISTS);

AddLinkedList0:
	pointers[i].inuse=true;
	pointers[i].head=0;
	pointers[i].tail=0;
	*list=i;
	state=CX_STATE_LIST;
	IterUnLock();			// once i set the flag true its mine
	if (loglevel>=4) ThreadLog("CXIterator:: AddLinkedList: Exit ok");
	return(ERR_NOERROR);

	} /////////////////////////////////////////////////////////////////////////end AddLinkedList

////////////////////////////////////////////////////////////////////////////////////////////////////
// DelLinkedList
// This will run through all the items reachable by headarray[list] and set the ptrs to 0.
// It will also set the headarray[list].ptr to 0 and set the inuse to false.
// Remember that there is no guarantee that all the items are in every list.
// RIGHT NOW THERE IS NO SUPPORT FOR NOT ORPHANING ITEMS.
// IT IS UP TO THE APP CODE TO ENSURE THAT WHEN A LIST IS DELETED, THAT THERE IS STILL A WAY TO ACCESS
// ALL ALLOCATED ITEMS.
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CXIterator::DelLinkedList(WIIILIST list)
{
CXItem	*p,*tp;

	// 
	IterLock();
	if (!pointers[list].inuse) {IterUnLock(); return false;}
	p=pointers[list].head;
	while (p)
	{
		tp=p;
		p=p->nextitem[list];
		tp->previtem[list]=0;
		tp->nextitem[list]=0;
		}
	pointers[list].head=0;
	pointers[list].tail=0;
	pointers[list].inuse=false;
	IterUnLock();
	return true;
	} /////////////////////////////////////////////////////////////////////////end DelLinkedList

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetCountOfLinkedLists
// Returns a count of lists in use.
////////////////////////////////////////////////////////////////////////////////////////////////////
int CXIterator::GetCountOfLinkedLists()
{
int 	i,result;

	result=0;

	// start the below loop from 1 because the "0" list is a hidden internal list
	// to help us to ensure we can delete ALL items upon destruction
	for (i=1;i<maxlistcount;i++) if (pointers[i].inuse) result++;
	return (result);
	} /////////////////////////////////////////////////////////////////end GetCountOfLinkedLists

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetLinkedLists
// returns an count sized array of WIIILIST items 
// This returns an array containing the handles to the lists that currently exist.
// The current lists may or may not be in numerical order, and will probably not be contiguous.
////////////////////////////////////////////////////////////////////////////////////////////////////
void CXIterator::GetLinkedLists(WIIILIST **resarray,int *count)
{
int	i,j,k;
WIIILIST	*l;

	// allocate return array, caller should delete when ready
	k=*count=GetCountOfLinkedLists();
	l = new WIIILIST[k];	// why in the HECK did i have to do it this way ?
	j=0;

	// add results
	// start the below loop from 1 because the "0" list is a hidden internal list
	// to help us to ensure we can delete ALL items upon destruction
	for (i=1;i<maxlistcount;i++) 
	{
		if (pointers[i].inuse) l[j++]=i; 
		}
	*resarray=l;
	} ////////////////////////////////////////////////////////////////////////end GetLinkedLists

////////////////////////////////////////////////////////////////////////////////////////////////////
// P_DeleteItem
// Does the actual delete.
////////////////////////////////////////////////////////////////////////////////////////////////////
//CXItem *CXIterator::P_DeleteItem(WIIIHANDLE curs, WIIILIST list, CXItem *item)
CXItem *CXIterator::P_DeleteItem(WIIIHANDLE curs, CXItem *item)
{
int	i;

	// now remove from each list
	for (i=0; i<maxlistcount; i++)
	{
//		printf("Before RemoveItemFromList i=%d \n",i);
		// new remove the item from each given list.
		if (pointers[i].inuse) RemoveItemFromList(i, item, curs);
		}

	// now delete item
	delete item;

	} //////////////////////////////////////////////////////////////////////////end P_DeleteItem

////////////////////////////////////////////////////////////////////////////////////////////////////
// DeleteItem
// Does some thread protection and locking then calls P_DeleteItem for actual delete.
// we do lazy delete 
// 1) Try to Write lock the item
// 2) If can then delete it
// 3) If cant, set delete flag and exit.
// 4) In cursor move routines check for delete flag to remove.
////////////////////////////////////////////////////////////////////////////////////////////////////
CXItem *CXIterator::DeleteItem(WIIIHANDLE curs)
{	
int		i;
int		result;
CXItem		*item;
WIIILIST	list;	
CXItem		*retval;

        // First provide external thread protection
        // Lock The top level iterator.
        IterLock();

        // Check cursor
        if (!CheckCursor(curs)) {IterUnLock(); return(0);}

	// Make it easier to read
	list=cursarray[curs].list;

	if (P_StdChecks(curs,list))
	{

		// prevent other list operations for a bit
		ListLock(list);

		// Let others back into the Iterator
		IterUnLock();

		// find pointer to item to delete
		item=cursarray[curs].curr;

		// Here we need to add in the "Lazy Delete"
		result=item->ItemTryWriteLock();

		if (result==0)		// 0 == it worked, we got the lock
		{

			// P_DeleteItem(RemoveItem) sets curr to next item or null as required
			retval=P_DeleteItem(curs,item);
			}
		else
		{
			// do "Lazy Delete"
			item->delflag=true;
	
			// not sure if this is a good idea yet, but we'll set curr to nextitem
			retval=P_MoveNext(curs,list);
			}
		ListUnLock(list);
		}
	else
	{
		IterUnLock();
		retval=0;
		}
	return(retval);
	} ////////////////////////////////////////////////////////////////////////////end DeleteItem

////////////////////////////////////////////////////////////////////////////////////////////////////
// RemoveItemFromList
// this adjusts links and removes item from list.
// It does not delete the item. If any particular cursor points to to the item being deleted 
// reset that cursor to 0.
////////////////////////////////////////////////////////////////////////////////////////////////////
CXItem *CXIterator::RemoveItemFromList(WIIILIST l, CXItem *item, WIIIHANDLE curs)
{
CXItem *n;
char	buf[200];

	// do some error checking
	// exactly one item in list
	if ((pointers[l].head==pointers[l].tail) && (item==pointers[l].head)) 
	{
		pointers[l].head=pointers[l].tail=0;
		cursarray[curs].curr=0;
		ThreadLog("CXIterator:: RemoveItemFromList: Deleted single item, returning.");
		return (0);
		}

	// first item in list
	if (pointers[l].head==item) 
	{
		pointers[l].head=pointers[l].head->nextitem[l];
		pointers[l].head->previtem[l]=0;
		cursarray[curs].curr=pointers[l].head;
		return(0);
		}

	// last item in list
	if (pointers[l].tail==item) 
	{
		pointers[l].tail=pointers[l].tail->previtem[l];
		pointers[l].tail->nextitem[l]=0;
		cursarray[curs].curr=0;
		return (0);
		}

	// item in middle
	// handle the case where the item is not in *this* list
//	snprintf(buf, 200, "Remove curs=%d list=%d val=%d prev=%lu next=%lu \n",
//			curs,l,item->value,item->previtem[l], item->nextitem[l]);
//	printf(buf);
	if ((item->previtem[l]==0) && (item->nextitem[l]==0))
	{
//		printf("BOTH NULL PTR\n");
		return(0);
		}
	if (item->nextitem[l]==0) { printf("NEXT NULL PTR\n"); return(0); }
	if (item->previtem[l]==0) { printf("PREV NULL PTR\n"); return(0); }

	// ok it *should* be in this list so delete it.
	n=item->previtem[l]->nextitem[l]=item->previtem[l]->nextitem[l]->nextitem[l];
	item->nextitem[l]->previtem[l]=item->nextitem[l]->previtem[l]->previtem[l];
	return (0);
//	if (loglevel>=4) ThreadLog("CXIterator:: DeleteItem: Unhandled delete condition in CXItem delete. ");

	} ////////////////////////////////////////////////////////////////////end RemoveItemFromList

////////////////////////////////////////////////////////////////////////////////////////////////////
// Virtual Functions
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetServerId
// Virtual
////////////////////////////////////////////////////////////////////////////////////////////////////
int CXItem::GetObjectId()
{ return (0);}

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetServerId
// Virtual
////////////////////////////////////////////////////////////////////////////////////////////////////
int CXItem::GetServerId()
{ return (0);}

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetRawRootPtr
// Virtual
////////////////////////////////////////////////////////////////////////////////////////////////////
//CRawBuf *CXItem::GetRawRootPtr()
//{ return (0);}

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetSockPtr
// Virtual
////////////////////////////////////////////////////////////////////////////////////////////////////
CUSock *CXItem::GetSockPtr()
{ return (0);}

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetCharacterPtr
// Virtual
////////////////////////////////////////////////////////////////////////////////////////////////////
CChar *CXItem::GetCharacterPtr()
{ return (0);}

////////////////////////////////////////////////////////////////////////////////////////////////////
// HarvestPortmon
// Virtual
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CXItem::HarvestPortmon()
{ return (0);}

////////////////////////////////////////////////////////////////////////////////////////////////////
// HarvestQLoader
// Virtual
////////////////////////////////////////////////////////////////////////////////////////////////////
//bool CXItem::HarvestQLoader()
//{ return (0);}

////////////////////////////////////////////////////////////////////////////////////////////////////
// HarvestServer
// Virtual
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CXItem::HarvestServer()
{ return (0);}

////////////////////////////////////////////////////////////////////////////////////////////////////
// HarvestServer
// Virtual
////////////////////////////////////////////////////////////////////////////////////////////////////
OBJIDTYPE CXItem::GetCharacterID()
{ return (0);}





